.\" ** You probably do not want to edit this file directly **
.\" It was generated using the DocBook XSL Stylesheets (version 1.69.1).
.\" Instead of manually editing it, you probably should edit the DocBook XML
.\" source for it and then use the DocBook XSL Stylesheets to regenerate it.
.TH "KEYVAL2JAVA" "l" "" "" ""
.\" disable hyphenation
.nh
.\" disable justification (adjust text to left margin only)
.ad l
.SH "NAME"
keyval2java \- Converts XML <keyval> files to Java code for configuration support.
.SH "SYNOPSIS"
.HP 10
\fBconfigure\fR [\fB\-n\fR\ \fITEXT\fR \fB\-\-name\fR\ \fITEXT\fR] [\fB\-\-reloc\-cfg\fR\ \fIFILENAME\fR] [\fB\-\-namespace\fR\ \fITEXT\fR] [\fB\-t\fR\ \fItrue|false\fR \fB\-\-templates\fR\ \fItrue|false\fR] [\fB\-l\fR\ \fIFILENAME\fR \fB\-\-log\-file\fR\ \fIFILENAME\fR] [\fB\-\-install\-query\fR\ \fITEXT\fR]
.SH "DESCRIPTION"
.PP
The
\fBkeyval2java\fR
treats a <keyval> file as a represention of configuration for a application. Using the entities defined in the <keyval>
XML
source file, it then creates the following:
.TP
\fINAME.java\fR
A
Java
class which can be used to access the configuration within a
Java
program.
.TP
\fINAMEValuesConfig.java\fR
A implementation of a
com.ccg.values.ValuesConfig
suitable for loading, saving, and listing user configurations.
.TP
\fINAMEValuesEditor.java\fR
A implementation of a
com.ccg.values.ValuesEditor
suitable for provideding a bare bones graphical interface for editing a configuration.
.TP
\fIhelp/NAME.html\fR
A
Java
help file. To provide on\-line help information for the graphical interface.
.TP
\fImsgs/NAME.properties\fR
A
Java
properties file. To provide the default language (text) to be used on the graphical interface.
.PP
The general goal is to reduce the amount of work required by a developer to maintain consistency in way a applications configuration is handled. By having all of these files tied to a single
XML
source file, consistency is guaranteed.
.PP
A tutorial for using this script is available in the
[1]\&\fIxml\-keyval\fR
package document. It will take you through the steps in creating a
Java
application.
.SH "OPTIONS"
.PP
The following command line options are available:
.TP
[\fB\-n TEXT|\fR\fB\-\-name TEXT\fR]
This utility is designed to work with a set of files all having a root NAME. This option is used to specify the root NAME. This utility assumes that NAME_config.xml will be the source XML file. This utility can then generate NAME_config.hh, NAME_config.cc, NAME_config.m4i, and NAME_config.cfg. In addition, when templates have not been disabled, it will also generate a template NAME_config.xml if it doesn't yet exist.
.TP
[\fB\-\-reloc\-cfg FILENAME\fR]
By default, a template configuration file is produced in the current directory with the name of NAME_config.cfg. Use this option, if you would like the template configuration file relocated to a different file location. Specify the full file name you want it to be placed under. For example
\-\-reloc\-cfg $APAHOME/src/package/sdsimu.cfg.
.TP
[\fB\-\-namespace TEXT\fR]
Use this option if you need to specify a namespace value other than the default of
lp. This would only be necessary when creating configuration definitions for code outside of the
GAIapa
package. The only well known namespace values at this time are
lp
and
nlib.
.TP
[\fB\-t true|false|\fR\fB\-\-templates true|false\fR]
This utility will stub in a template file to get you off on the right foot. Unless you disable this feature, it will generate a template version of NAME_config.xml. However, this utility never replaces NAME_config.xml if it already exists.
.TP
[\fB\-l FILENAME|\fR\fB\-\-log\-file FILENAME\fR]
If you are encountering errors when running
\fBkeyval2cc\fR, try using this option and examine the log file after running
\fBkeyval2cc\fR. It may provide useful clues as to what went wrong.
.TP
[\fB\-\-install\-query TEXT\fR]
This option allows a user to specify options like
\-\-install xsl
to retrieve information about the package installation. This might be useful for making use of the package files outside of the packaged scripts.
\-\-install help
will list all possible. The following choices are recognized:
help
\- lists what choices are available.
xsl
\- shows the directory the XSL files are stored under.
public\-id
\- displays the public ID of the keyval documents.
public\-url
\- displays the public URL of the keyval DTD.
dtd
\- shows the location of the keyval DTD file.
catalog
\- shows the location of the XML style catalog file.
catalog\-oasis
\- shows the location of the OASIS style catalog.
docs
\- shows local directory where the documents can be found.
url
\- shows the public URL where documenation can be found.
.SH "FILES"
.TP
\fI/opt/VAIxml/share/vaisala/xml/keyval/dtd/keyval_1_0_0.dtd\fR
Location of the
DTD
which is used to validate the <keyval> document.
.TP
\fI/opt/VAIxml/share/vaisala/xml/keyval/xsl\fR
The directory containing the
XSL
files used when translating <keyval> documents.
.TP
\fI/opt/VAIxml/share/doc/keyval\fR
The directory containing the package documentation (including a tutorial on using this utility).
.SH "ENVIRONMENT"
.TP
\fBXML_COMMON_JAVA\fR
When translating
XML
documents with a
XSL
style sheet, the
\fBkeyval2java\fR
will make use of the
XSLT
processor found in the 1.4.2 version of the Java run time environment. If the
\fBjava\fR
interpreter found in your path is not version 1.4.2 or later, you can use this environment variable to specify the location of the appropriate
\fBjava\fR
interpreter. In addition, you can also use it to specify
HTTP
proxy information for your
\fBjava\fR
environment. For example, a setting of
/liz3/pkb/usr/j2re1.4.2_03/bin/java \-DproxyHost=172.26.0.232 \-DproxyPort=3128
would specify a path to a particular
\fBjava\fR
interpreter and set the
HTTP
proxy settings.
.SH "AUTHORS"
.PP
Paul initial script,
XSL
files, and build environment.
.SH "SEE ALSO"
.PP
\fBkeyval2cc\fR(l),
\fBkeyval2sh\fR(l),
[1]\&\fIxml\-keyval\fR
package.
.SH "AUTHOR"
Paul Blankenbaker <paul@mekwin.com>. 
.SH "REFERENCES"
.TP 3
1.\ xml\-keyval
\%/VAIxml/keyval/index.html
